;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x065f7a82499ba361954a2c865af7e014 . "./code.cg2js/cg-status-bar.fasl")
(#xd34804d4a655b0ee99b96d405c3c73a6 . "./code.cg2js/cg-hotspot.fasl")
(#x5e5f5c622488e19a98f7567674c78b5f . "./code.cg2js/cg-toolbar.fasl")
(#xb84c253efbde578fd4afdb4adfe400d3 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xf601cb25e7f93f61fe4437e5a956876d . "./code.cg2js/cg-tooltip.fasl")
(#xdcac4b1a403a1e247a286249a882ff51 . "./code.cg2js/cg-calendar.fasl")
(#x410c3ec33db0d3ca01c9a983e52491fa . "./code.cg2js/cg-item-list.fasl")
(#x6c533a2cc6b1c7183e0640774293d777 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xe4ae8430b01e94712200b08af421d446 . "./code.cg2js/cg-trackbar.fasl")
(#x95a74d9e21f0be3232244588d5122e15 . "./code.cg2js/cg-class-grid.fasl")
(#x71aae5a454a59c397c8498bd4f960f47 . "./code.cg2js/cg-dialog-item.fasl")
(#x3ad2f867b45c0869fa370f4dc07c1305 . "./code.cg2js/cg-comtab.fasl")
(#x58d437f25e440f01ef4c64d7d1adb839 . "./code.cg2js/cg-lisp-text.fasl")
(#xc90482e2a770dee5183bd834d8b563e7 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xa7e7f791df7b1ce29044296c2080e233 . "./code.cg2js/cg-button.fasl")
(#x884eeffa9bc51ad50290dbc014962e15 . "./code.cg2js/cg-printing.fasl")
(#x19dda9260cd45387d2c8c5fcae762db4 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xf6d9bf0da5e2f3f8a70bf9abfb69f598 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xcfd365c87b8631a5b263f1dec8d6ee4d . "./code.cg2js/cg-pixmap-widget.fasl")
(#xdbb5864ba7d625cf02f460a648fbde44 . "./code.cg2js/cg-selected-object.fasl")
(#xd83a953ac913b71c4752e48040b6f38f . "./code.cg2js/cg-text-or-combo.fasl")
(#x8e95d53842100e87c9a1b6b9fc7da6ba . "./code.cg2js/cg-graphics-context.fasl")
(#x65fc66b9e895c3efe0a2612e79d22347 . "./code.cg2js/cg-edit-in-place.fasl")
(#xd23edfdecbc2c53a9a257ea50cd8494b . "./code.cg2js/cg-string-dialog.fasl")
(#xe87055d7a00aca20909c0299b9431d8f . "./code.cg2js/cg-list-view.fasl")
(#x0267a2b814a976e37d0d9d54a281d3df . "./code.cg2js/cg-intersect.fasl")
(#x131b915697ae42b4ebf11442215069ad . "./code.cg2js/cg-text-widget.fasl")
(#x074fa03ef6288ecc0fc9b1611d781420 . "./code.cg2js/cg-directory-dialog.fasl")
(#xa7c2b71fa1b406d59dbb6e115ce28b98 . "./code.cg2js/cg-file-dialog.fasl")
(#xf8ade5aa7877afd385038d47fafdfed2 . "./code.cg2js/cg-dde-utils.fasl")
(#x5609153d3893b4d55f39707980c9eadb . "./code.cg2js/cg-nodes.fasl")
(#x72303e03ab39c952772367417d2b2d9f . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x3d6c545f26d9d969108c7234621852e2 . "./code.cg2js/cg-up-down-control.fasl")
(#xb771d680dfa0a0bd0e49638d8cb7e67f . "./code.cg2js/cg-get-position.fasl")
(#x4fc8e1b99a3950425c9ac7d7a3376b70 . "./code.cg2js/cg-toggling-widget.fasl")
(#xdd8b73f6c34f6a68b0914cbae44d5dfe . "./code.cg2js/cg-acache.fasl")
(#xb758afb8aa54f646002795c377eb23aa . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x571a3578d9889bf102ec0e22485dec7c . "./code.cg2js/cg-base.fasl")
(#x5c840fd1ab3e0ae9f4906481160115d5 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x9dff9027794861270b5ecf6e6a58cee5 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x8960c66828470bd6e3085fd7a09a2e55 . "./code.cg2js/cg-property.fasl")
(#x34b86124ebb11133a5eda4a7da871012 . "./code.cg2js/cg-editable-text.fasl")
(#x30c1967c383a884cb544943bac2782cf . "./code.cg2js/cg-lamp.fasl")
(#x98123c887ff5f0644ba1e3e6ef056d91 . "./code.cg2js/cg-progress-indicator.fasl")
(#x85a703deb1f6179c01c2b7bdebbd1086 . "./code.cg2js/cg-chart-widget.fasl")
(#x7b2f84c2f5618c1be3915251c61cad40 . "./code.cg2js/cg-common-control.fasl")
(#xaa01ca441765f251e959a179b8c5c026 . "./code.cg2js/cg-font-dialog.fasl")
(#x8f793d285ee83e1e162a438d7ae2093a . "./code.cg2js/cg-gesture-emulation.fasl")
(#x29dfc518e61bc9f11b0015aecb44d9de . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x55c4456339573b4837f20f951fee1532 . "./code.cg2js/cg-os-window.fasl")
(#x2d1858d5bb9a41f7ec1f4d8ba2b1d1dd . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xdf0134c92b00a1c003dad5f81707b1c3 . "./code.cg2js/cg-pixmap.fasl")
(#x8b26b2466ec5f4b81ffdd9fe9182ee10 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x19cc1ccea9a7ad11f607ec8101c09cf6 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x55bd480e6f2b3ed02fc1ee11373ed175 . "./code.cg2js/cg-scaling-stream.fasl")
(#x062fd8fa321d8f62b6d5bacb49b55ac1 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x59c740639af7c135bf8016b09a944fdb . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x9bf1172fc9deb6421daa6e4b62be7a14 . "./code.cg2js/cg-timer.fasl")
(#x95fc5f1503664f6d4b6b98a9b9fcb754 . "./code.cg2js/cg-scroll-bar.fasl")
(#xc962426419c3d6196af964c6fc2cf97b . "./code.cg2js/cg-caret.fasl")
(#x51a97733ee83852469eb811073246cda . "./code.cg2js/cg-template-string.fasl")
(#x88e76920db99567b027282e19a72367d . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xebf366ebb903e72a420266102d4f1e98 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x749ff266b8474327068f4a7243d12123 . "./code.cg2js/cg-rich-edit.fasl")
(#x9248b90c52880129e8cb6d035f95e46f . "./code.cg2js/cg-combo-box.fasl")
(#x3a821d2a8748a3e49ab25143a4a75259 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x2628908dc46fa7f23024647bc0df9c91 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xc93c4cc7b1ebf87333f40526cbb5c725 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xf02c04a612bc256ab33e8e08980f7b78 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xf47a860ff9b5eb4a353f8c8fb31ae72b . "./code.cg2js/cg-outline.fasl")
(#x22930691c7ed1dd636bf860bcf9451dd . "./code.cg2js/cg-class-support.fasl")
(#x018b292fa3ef2fef00df0184bc59bfa7 . "./code.cg2js/cg-lettered-menu.fasl")
(#x7eb20c234ff7be9451a0656bc9d2b0b4 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x4c8e3abd9e3031da44e1c22424a44b36 . "./code.cg2js/cg-fill-texture.fasl")
(#x75eb4df7a07dc61db4cec04b761e64cd . "./code.cg2js/cg-lisp-widget.fasl")
(#x4cd102b58313bd56abc051c411547129 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x29fdc88a0aa6b539e90f6d25389936de . "./code.cg2js/cg-clipboard.fasl")
(#x17d501151573e3b0baf81945f42dc0a3 . "./code.cg2js/cg-wrap-string.fasl")
(#xa898d77bce0ff9b714fdacf6b0e9c8ab . "./code.cg2js/cg-os-widget.fasl")
(#xa5a8a1bdbb3be983de715ad792e6e00e . "./code.cg2js/cg-check-box.fasl")
(#xddc3c8edd7a8ddb7e3a29431e8f9c610 . "./code.cg2js/cg-choice-list.fasl")
(#x4908440015aa98d4391133bb8f07793c . "./code.cg2js/cg-layout.fasl")
(#x15aa981245af1b87d6c9bddd9a3d11ea . "./code.cg2js/cg-group-box.fasl")
(#xe2f910aa9c667226c927b41eb265cb93 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x6a236f58f7dc5635f9bcc9e6eead840a . "./code.cg2js/cg-radio-button.fasl")
(#xed32c53ed233aecba78c4c406a8ec2ef . "./code.cg2js/cg-tab-control.fasl")
(#x9c6443483f5b2c538524b347f08815e0 . "./code.cg2js/cg.fasl")
(#xdd2d1c450259a2d9bcb181b5ce24d71b . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xf67e220bf698dc45417210c6731bbe3b . "./code.cg2js/cg-dropping-outline.fasl")
(#x81b2e938e2e3191afa73f4b876649244 . "./code.cg2js/cg-get-pixmap.fasl")
(#x1cfe8f3d138686c4ca0b57a4c2f7be48 . "./code.cg2js/cg-grid-widget.fasl")
(#xc093db0505b61192177d588c052322f3 . "./code.cg2js/cg-utility-dialog.fasl")
(#x04f5a477cf3a1522286e24bcd726b6d8 . "./code.cg2js/cg-static-text.fasl")
(#xaaec73bd2f25e9ca03235f7e427cb6fa . "./code.cg2js/cg-class-slot-grid.fasl")
(#x3f23c8acb9ce955204c28fd2fd22e9bb . "./code.cg2js/cg-object-editor-layout.fasl")
(#xb05c02b3927d874569a179e1adb86194 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xa8305970474c067d1ef6ed04223d0861 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x3b9865dc70d1c156278d00877af995f7 . "./code.cg2js/cg-plot-widget.fasl")
(#x6b8a0e8d37e2de838b192fab952ef44f . "./code.cg2js/cg-color-dialog.fasl")
(#x479a9f7c1118a829a4748d5aa99b43ed . "./code.cg2js/cg-icon-pixmap.fasl")
(#xbea5c285f10ae596a5132612407b85fb . "./code.cg2js/cg-icon.fasl")
(#x313e56f055c4aa5d269d74a0c1d85a16 . "./code.cg2js/cg-dev.fasl")
(#xdd9e4aa2b7611df21821d91e89a97463 . "./code.cg2js/cg-choose-printer.fasl")
(#x48f72ec30b0d3a7c78359bca8496985e . "./code.cg2js/cg-intersect-posbox.fasl")
(#xe79d4b274c8ab09a0ea5b10e279b5f31 . "./code.cg2js/cg-menu.fasl")
(#x6407a39b1e4e2f27d64d86b1c06e64da . "./code.cg2js/cg-yes-no-list.fasl")
(#x1886a0a22302164c27ec73264142fd58 . "./code.cg2js/cg-palette.fasl")
(#x17ac22aeae6217f213fc96f9c498a685 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x5ab3e13c06b3f6f1d7dfc0af4a7d70a5 . "./code.cg2js/cg-message-dialog.fasl")
(#x7dc5f11998a9b53eb5cfd831564f7426 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x1cc733a11568e32764803db56c1bae1d . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xe2232ec108671ea1eead884ff057f2bd . "./code.cg2js/cg-split-bar.fasl")
(#x41505405e90d6ce68600e6f2676a4192 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xf253b8db8d1bd2b71365b6ff79f7b05c . "./code.cg2js/cg-web-browser.fasl")
(#x4a240a7314e7d8b90efdaedb9e087453 . "./code.cg2js/cg-project-window.fasl")
(#x8cfd9ce005e88990a73fd401a694cc4e . "./code.cg2js/cg-picture-widget.fasl")
(#x33307997703d05568dbf498dce862430 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x4d0c50763b53745c71084bb062b3b00e . "./code.cg2js/cg-object-editor.fasl")
(#x96766b8e1d9cf411d060c75ac4c75fbc . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xc29bcc97bc8d80c40701136e0589dbd3 . "./code.cg2js/cg-paren-matching.fasl")
(#x9262262bcbad433d11ed3a89a4e1afee . "./code.cg2js/cg-sample-file-menu.fasl")
(#x648cef956b49c614335e948586e74822 . "./code.cg2js/cg-drawable.fasl")
(#x5be13f6d9b887e5491bb0bf18cc3d020 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x80d71bfa59dc2d48b649c8b5d256814e . "./code.cg2js/cg-html-widget.fasl")
(#x4cd5cf5484106862c5b2516b5bac5d96 . "./code.cg2js/cg-curve.fasl")
(#x33f610cd4a9b31679adfc41d5246f84a . "./code.cg2js/cg-yes-no-string.fasl")
(#x8a839c70307952ea9827cd489ce3e75e . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xc3b05e3406d057513dfeec03a0d871c4 . "./code.cg2js/cg-html-dialog.fasl")
(#xb99e47f659b15543b7b64d65e24233d4 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x67237ca9acb3d65443a1eee18083b20b . "./code.cg2js/cg-tray.fasl")
(#xd540d9055daff62d8b2b88756ddd0ef2 . "./code.idejs/ide-apropos.fasl")
(#x919be94e2e8b51cfd09ae97c06f7ba77 . "./code.idejs/ide-window-list.fasl")
(#x732182b19ae0f3823a401f7519047fb9 . "./code.idejs/ide-class-browser.fasl")
(#x29f288ca451720633843416e0b775798 . "./code.idejs/ide-base.fasl")
(#xe6d831065e0fde8463490ab7a09080fa . "./code.idejs/ide-process-browser.fasl")
(#xe606b48c5744270ad861743e3d730b96 . "./code.idejs/ide-acache.fasl")
(#x6522cfee916085383dbca4459a0ad061 . "./code.idejs/ide-navigator.fasl")
(#x2839456f201c1fa00816a7432fea0460 . "./code.idejs/ide-trace.fasl")
(#xc838dd1f16f466daa51cff50d4eed2e4 . "./code.idejs/ide-project.fasl")
(#x0d86759589f518285b9c225c91cd1c8e . "./code.idejs/ide-patch-dialog.fasl")
(#x4752e0053675b009b4f25926eef8d662 . "./code.idejs/ide-grapher.fasl")
(#xf2c42d33bc2675d5fbb720f1aa55f684 . "./code.idejs/ide-dev.fasl")
(#xd1f009c04480162d8423c584b506960d . "./code.idejs/ide-keyshort.fasl")
(#x933e158cdb4fa60c50d3d66ad17d7e3c . "./code.idejs/ide-editor.fasl")
(#x1b90fdbb81cad7ac5663d579dbbaa52f . "./code.idejs/ide-inspector.fasl")
(#x82fecb30a9b69c91179799d7b72fd09a . "./code.idejs/ide.fasl")
(#x8afa4d4f971ff4180ece452c405de7db . "./code.idejs/ide-clipboard.fasl")
(#xd7b4f791405089c765d6ceb06f045f19 . "./code.idejs/ide-find-definitions.fasl")
(#x9fbeea67beea4cf8cd821e10ac740730 . "./code.idejs/ide-form.fasl")
(#x5f23d26c5c82b6cddde52d041177a0f9 . "./code.idejs/ide-intro.fasl")
(#x5374e26875ce9a400e6bfed4d08645a5 . "./code.idejs/ide-find-in-files.fasl")
(#xafe9fd8da35905527b1f83ad6999e4f9 . "./code.idejs/ide-package-list.fasl")
(#x9e7a2fe9c2d2b1a0a09ce0794a64ec54 . "./code.idejs/ide-class-grapher.fasl")
(#x49a17b9a618997e974ff2e9af0f2eb06 . "./cgjs.html")
(#x44f169492307df5920f609d11cc1a34c . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
